.hll {
    background-color: #ffffcc
}

.codehilite {
    background: oklch(var(--b3)/1);
}

.c {
    color: #008000
}

/* Comment */
.err {
    border: 1px solid #FF0000
}

/* Error */
.k {
    color: var(--color-prettylights-syntax-markup-heading)
}

/* Keyword */
.ch {
    color: #008000
}

/* Comment.Hashbang */
.cm {
    color: #008000
}

/* Comment.Multiline */
.cp {
    color: var(--color-prettylights-syntax-markup-heading)
}

/* Comment.Preproc */
.cpf {
    color: #008000
}

/* Comment.PreprocFile */
.c1 {
    color: #008000
}

/* Comment.Single */
.cs {
    color: #008000
}

/* Comment.Special */
.ge {
    font-style: italic
}

/* Generic.Emph */
.gh {
    font-weight: bold
}

/* Generic.Heading */
.gp {
    font-weight: bold
}

/* Generic.Prompt */
.gs {
    font-weight: bold
}

/* Generic.Strong */
.gu {
    font-weight: bold
}

/* Generic.Subheading */
.kc {
    color: var(--color-prettylights-syntax-markup-heading)
}

/* Keyword.Constant */
.kd {
    color: var(--color-prettylights-syntax-markup-heading)
}

/* Keyword.Declaration */
.kn {
    color: var(--color-prettylights-syntax-markup-heading)
}

/* Keyword.Namespace */
.kp {
    color: var(--color-prettylights-syntax-markup-heading)
}

/* Keyword.Pseudo */
.kr {
    color: var(--color-prettylights-syntax-markup-heading)
}

/* Keyword.Reserved */
.kt {
    color: #2b91af
}

/* Keyword.Type */
.s {
    color: var(--color-prettylights-syntax-keyword)
}

/* Literal.String */
.nc {
    color: #2b91af
}

/* Name.Class */
.ow {
    color: var(--color-prettylights-syntax-markup-heading)
}

/* Operator.Word */
.sa {
    color: var(--color-prettylights-syntax-keyword)
}

/* Literal.String.Affix */
.sb {
    color: var(--color-prettylights-syntax-keyword)
}

/* Literal.String.Backtick */
.sc {
    color: var(--color-prettylights-syntax-keyword)
}

/* Literal.String.Char */
.dl {
    color: var(--color-prettylights-syntax-keyword)
}

/* Literal.String.Delimiter */
.sd {
    color: var(--color-prettylights-syntax-keyword)
}

/* Literal.String.Doc */
.s2 {
    color: var(--color-prettylights-syntax-keyword)
}

/* Literal.String.Double */
.se {
    color: var(--color-prettylights-syntax-keyword)
}

/* Literal.String.Escape */
.sh {
    color: var(--color-prettylights-syntax-keyword)
}

/* Literal.String.Heredoc */
.si {
    color: var(--color-prettylights-syntax-keyword)
}

/* Literal.String.Interpol */
.sx {
    color: var(--color-prettylights-syntax-keyword)
}

/* Literal.String.Other */
.sr {
    color: var(--color-prettylights-syntax-keyword)
}

/* Literal.String.Regex */
.s1 {
    color: var(--color-prettylights-syntax-keyword)
}

/* Literal.String.Single */
.ss {
    color: var(--color-prettylights-syntax-keyword)
}

/* Literal.String.Symbol */